local _, core = ...; -- Namespace

local SLOT_NAMES = {
	"HEADSLOT",
	"NECKSLOT",
	"SHOULDERSLOT",
	"BACKSLOT",
	"CHESTSLOT",
	"SHIRTSLOT",
	"TABARDSLOT",
	"WRISTSLOT",
	"HANDSSLOT",
	"WAISTSLOT",
	"LEGSSLOT",
	"FEETSLOT",
	"FINGER0SLOT",
	"FINGER1SLOT",
	"TRINKET0SLOT",
	"TRINKET1SLOT",
	"MAINHANDSLOT",
	"SECONDARYHANDSLOT"
}

local GBANK_TABS = 8
local GBANK_SLOTS = 98
local SOULS_ID = 1810
local CURRENT_VERSION = 2

--------------------------------------
-- Defaults (usually a database!)
--------------------------------------
local defaults = {
	theme = {
		r = 0, 
		g = 0.8, -- 204/255
		b = 1,
		hex = "00ccff"
	}
}

--------------------------------------
-- Config functions
--------------------------------------

function GetThemeColor()
	local c = defaults.theme
	return c.r, c.g, c.b, c.hex
end

function core:Print(...)
	local stringInput = tostring(...)
    local hex = select(4, self.AltVault:GetThemeColor());
    local prefix = string.format("|cff%s%s|r", hex:upper(), "AltVault:");	
    DEFAULT_CHAT_FRAME:AddMessage(string.join(" ", prefix, stringInput));
end

local function OpenAltVault()	
	AltVaultFrame:Show()
end

function charInDB(charGUID)
	if AltVaultDB == nil then return end
	for index, character in pairs(AltVaultDB.characters) do
		if character.GUID == charGUID then
			return index
		end
	end
	return false;
end

--------------------------------------
-- Init function
--------------------------------------

function core:initDB()
	-- data reset for major version change
	if not AltVaultDB.data.version or AltVaultDB.data.version ~= CURRENT_VERSION then
		AltVaultDB.characters = {}
		AltVaultDB.data.version = CURRENT_VERSION
		StaticPopup_Show("NEW_VERSION_RESET")
	end

	-- one time remove bank data to update search terms
	if AltVaultDB.data.searchable == nil and AltVaultDB.characters ~= nil then
		resetBankData()
	end

	if AltVaultDB.data.hideGoldValues == nil then
		AltVaultDB.data["hideGoldValues"] = false 
	end
	
	local charGUID = UnitGUID("player");
	local dbIndex = charInDB(charGUID);
	local charIndex
	if not dbIndex then
		charIndex = #AltVaultDB.characters + 1;
		AltVaultDB.characters[charIndex] = {};
	else
		charIndex = dbIndex
	end

	local hasMail = false --type(latestMail)
	local title = GetTitleName(GetCurrentTitle());

	local gender = UnitSex("player");
	gender = (gender == 2) and "male" or "female";

	local raceName, raceKey = UnitRace("player");
	local className, classKey = UnitClass("player");
	local specID, specName, _, specBackground, role = GetSpecializationInfo(GetSpecialization(), false, false, false);
	local restStateName = select(2, GetRestState())
	local restedXP = GetXPExhaustion();

	if role == nil then
		-- this is nil sometimes for some reason??
		role = "DAMAGER"
	end

	local currentTime = C_DateAndTime.GetCurrentCalendarTime()
	
	AltVaultDB.characters[charIndex]["GUID"] = charGUID;
	AltVaultDB.characters[charIndex]["character"] = {
		["faction"] = UnitFactionGroup("player"),
		["name"] = UnitName("player"),
		["realm"] = GetRealmName(),
		["level"] = UnitLevel("player"),
		["race"] = raceName,
		["raceKey"] = raceKey,
		["class"] = className,
		["classKey"] = classKey,
		["specID"] = specID,
		["specName"] = specName,
		["specBackground"] = specBackground,
		["role"] = role,
		["sex"] = gender,
		["title"] = title,
		["lastPlayed"] = currentTime,
		["xp"] = UnitXP("player"),
		["xpMax"] = UnitXPMax("player"),
		["restState"] = restStateName,
		["restedXP"] = restedXP,
		["money"] = GetMoney(),
		["ilvl"] = 0,
		["ilvlColor"] = {}
	};
	if not dbIndex then
		AltVaultDB.characters[charIndex]["character"]["bankWarned"] = false
	end
	AltVaultDB.characters[charIndex]["gear"] = GetGearInfo()
	setItemLevelInfo(charIndex)
	RequestTimePlayed()
	local currentHonor = UnitHonor("player");
	local maxHonor = UnitHonorMax("player");
	local honorLevel = UnitHonorLevel("player");
	local tierID = C_PvP.GetSeasonBestInfo();
	local tierInfo = C_PvP.GetPvpTierInfo(tierID);
	local tierIcon = tierInfo.tierIconID;
	local currentConquest, maxConquest = PVPGetConquestLevelInfo();

	AltVaultDB.characters[charIndex]["pvp"] = {
		["honor"] = currentHonor,
		["kills"] = GetPVPLifetimeStats(),
		["maxHonor"] = maxHonor,
		["honorLevel"] = honorLevel,
		["tierID"] = tierID,
		["tierIcon"] = tierIcon,
		["currentConquest"] = currentConquest,
		["maxConquest"] = maxConquest
	}

	if AltVaultDB.characters[charIndex]["bags"] == nil then
		AltVaultDB.characters[charIndex]["bags"] = {}
	end
	if AltVaultDB.characters[charIndex]["bank"] == nil then
		AltVaultDB.characters[charIndex]["bank"] = {
			["bag-14"] = {}, -- placeholder for bank
			["bag-15"] = {} -- placeholder for reagents
		}
	end
	for bagID = 0, NUM_BAG_SLOTS do
		if AltVaultDB.characters[charIndex].bags['bag-'..bagID+1] == nil then
			AltVaultDB.characters[charIndex].bags['bag-'..bagID+1] = {}
		end
		GetBagInfo(bagID, charIndex)
	end

	if AltVaultDB.characters[charIndex]["professions"] == nil then
		AltVaultDB.characters[charIndex]["professions"] = {}
	end
	
	local prof1, prof2, arch, fish, cook = GetProfessions();
	local professionData = {
		["prof1"] = prof1,
		["prof2"] = prof2,
		["arch"] = arch,
		["fish"] = fish,
		["cook"] = cook
	}
	for key, prof in pairs(professionData) do
		if prof ~= nil then
			local name, texture, rank, maxRank, numSpells, spelloffset, _, rankModifier, _, _, skillLineName = GetProfessionInfo(prof);
			local tradeSkillLink, tradeSkillSpellID = GetSpellTradeSkillLink(prof, SpellBookFrame.bookType);
			if ( tradeSkillSpellID ) then
				core:Print(tostring(tradeSkillLink))
			else
				local spellLink = GetSpellLink(slot, SpellBookFrame.bookType);
			end
			
			local prof_title = "";
			if (skillLineName) then
				prof_title = skillLineName;
			else
				for i=1,#PROFESSION_RANKS do
					local value,title = PROFESSION_RANKS[i][1], PROFESSION_RANKS[i][2];
					if maxRank < value then break end
					prof_title = title;
				end
			end
			local specializationName
			if (name == "Alchemy" or name =="Engineering") then
				local spellName, _, spellID = GetSpellBookItemName(numSpells + spelloffset, "professions");
				specializationName = spellName
			end
			AltVaultDB.characters[charIndex].professions[key] = {
				["name"] = name,
				["skillName"] = prof_title,
				["icon"] = texture,
				["skill"] = rank,
				["maxSkill"] = maxRank,
				["skillModifier"] = rankModifier,
				["specializationName"] = specializationName,
			}
		end
	end

	AltVault_setupMinimap()
	AltVault_OnLoad()
end

--------------------------------------
-- One time update to add search terms
--------------------------------------

function resetBankData()
	local characters = AltVaultDB.characters
	for charIndex, character in ipairs(characters) do
		AltVaultDB.characters[charIndex]["bags"] = {}
		local bank = character.bank
		if bank ~= nil then
			AltVaultDB.characters[charIndex]["bank"] = {
				["bag-14"] = {}, -- placeholder for bank
				["bag-15"] = {} -- placeholder for reagents
			}
		end
	end
	AltVaultDB.data["searchable"] = true
end

--------------------------------------
-- MiniMap Button
--------------------------------------

function AltVault_setupMinimap()
	if not AltVaultDB.minimap then
		AltVaultDB.minimap = {
			profile = {
				minimap = {
					hide = false, 
				},
			},
		}
	end

	local dataObject = LibStub("LibDataBroker-1.1"):GetDataObjectByName("AltVault");

	if dataObject == nil then
		dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("AltVault", {
			type = "launcher",
			name = "AltVault",
			icon = "Interface\\Icons\\ACHIEVEMENT_GUILDPERK_EVERYONES A HERO_RANK2",
			OnTooltipShow = function(tooltip)
				tooltip:SetText("AltVault v2.0.1")
				tooltip:AddLine("Click to view character data.", 1, 1, 1)
				tooltip:Show()
		   end,
			OnClick = function(self, button)
				AltVaultFrame:SetShown(not AltVaultFrame:IsShown())
			end,
		})

		LibStub("LibDBIcon-1.0"):Register("AltVault", dataObject,  AltVaultDB.minimap)
	end
	

	if AltVaultDB.minimap.profile.minimap.hide == false then 
        LibStub("LibDBIcon-1.0"):Show("AltVault")
    else
        LibStub("LibDBIcon-1.0"):Hide("AltVault")
    end
end

--------------------------------------
-- Set Item Level
--------------------------------------

function setItemLevelInfo(charID)
	local itemLevel = select(2, GetAverageItemLevel())
	local ilvlR, ilvlG, ilvlB = GetItemLevelColor()
	local colors = {
		["red"] = ilvlR, 
		["green"] = ilvlG,
		["blue"] = ilvlB
	}
	updatePlayerData(charID, 'character', 'ilvl', floor(itemLevel))
	updatePlayerData(charID, 'character', 'ilvlColor', colors)
end

--------------------------------------
--Data Fetch functions
--------------------------------------

function GetGearInfo()
	local gear = {};
	for i, slot in ipairs(SLOT_NAMES) do
		local slotID = GetInventorySlotInfo(slot);
		local equippedItem = GetInventoryItemID("player", slotID);
		if equippedItem then
			local itemLink = GetInventoryItemLink("player", slotID);
			local itemString = select(3, strfind(itemLink, "|H(.+)|h"));
			equippedItem = itemString;
		end
		gear[slot] = equippedItem;
	end
	return gear
end

function GetBagInfo(bagID, charIndex)
	if charIndex == nil then return end
	local bagKey = 'bag-'..bagID+1
	local typeKey = "bags"

	local numSlots =  GetContainerNumSlots(bagID)
	local numFreeSlots = GetContainerNumFreeSlots(bagID)
	if bagID == -1 then -- primary bank slot
		bagKey = "bag-14"
		typeKey = "bank"
		AltVaultDB.characters[charIndex][typeKey][bagKey]["icon"] = "Interface\\Buttons\\Button-Backpack-Up"
	elseif bagID == -3 then -- reagent bank slot
		bagKey = "bag-15"
		typeKey = "bank"
		AltVaultDB.characters[charIndex][typeKey][bagKey]["icon"] = "Interface\\ICONS\\ACHIEVEMENT_GUILDPERK_BOUNTIFULBAGS"
	elseif bagID == 0 then	-- Bag 0	
		AltVaultDB.characters[charIndex][typeKey][bagKey]["icon"] = "Interface\\Buttons\\Button-Backpack-Up"
	else						-- Bags 1 through 11
		if bagID > NUM_TOTAL_EQUIPPED_BAG_SLOTS then
			typeKey = "bank"
		end
		AltVaultDB.characters[charIndex][typeKey][bagKey]["icon"] = GetInventoryItemTexture("player",  ContainerIDToInventoryID(bagID))
		bagLink = GetInventoryItemLink("player",  ContainerIDToInventoryID(bagID))
		AltVaultDB.characters[charIndex][typeKey][bagKey]["link"] = bagLink
		if bagLink then
			local _, _, rarity = GetItemInfo(bagLink)
			if rarity then	-- in case rarity was known from a previous scan, and GetItemInfo returns nil for some reason .. don't overwrite
				AltVaultDB.characters[charIndex][typeKey][bagKey]["rarity"] = rarity
			end
		end
	end

	AltVaultDB.characters[charIndex][typeKey][bagKey]["slots"] = numSlots
	AltVaultDB.characters[charIndex][typeKey][bagKey]["freeSlots"] = numFreeSlots
	if AltVaultDB.characters[charIndex][typeKey][bagKey]["items"] == nil then
		AltVaultDB.characters[charIndex][typeKey][bagKey]["items"] = {}
	end
	if AltVaultDB.characters[charIndex][typeKey][bagKey]["counts"] == nil then
		AltVaultDB.characters[charIndex][typeKey][bagKey]["counts"] = {}
	end
	for itemID = 1, numSlots do
		local iconFileID, stackCount, _, quality, _, _, hyperlink = GetContainerItemInfo(bagID, itemID);
		if iconFileID ~= nil then
			local searchName
			if hyperlink ~= nil then
				searchName = GetItemInfo(hyperlink)
			end
			AltVaultDB.characters[charIndex][typeKey][bagKey]["items"][itemID] = {
				["link"] = hyperlink,
				["icon"] = iconFileID,
				["rarity"] = quality,
				["searchName"] = searchName
			}
			if stackCount and stackCount > 1 then
				AltVaultDB.characters[charIndex][typeKey][bagKey]["counts"][itemID] = stackCount
			else
				AltVaultDB.characters[charIndex][typeKey][bagKey]["counts"][itemID] = nil
			end
		end
	end
end

function getGuildBankInfo(charIndex)
	local gBank = AltVaultDB.characters[charIndex]["gbank"]
	local tabID = GetCurrentGuildBankTab()
	local tabName, tabIcon = GetGuildBankTabInfo(tabID)

	if gBank["tab-"..tabID] == nil then
		gBank["tab-"..tabID] = {}
	end

	gBank["tab-"..tabID]["name"] = tabName
	gBank["tab-"..tabID]["icon"] = tabIcon
	if gBank["tab-"..tabID]["items"] == nil then
		gBank["tab-"..tabID]["items"] = {}
	end
	if gBank["tab-"..tabID]["counts"] == nil then
		gBank["tab-"..tabID]["counts"] = {}
	end
	local freeSlots = GBANK_SLOTS
	for slotId = 1, GBANK_SLOTS do
		local itemIcon, itemCount, _, _, rarity = GetGuildBankItemInfo(tabID, slotId);
		local itemLink = GetGuildBankItemLink(tabID, slotId)
		if itemLink ~= nil then
			local searchName = GetItemInfo(itemLink)
			gBank["tab-"..tabID]["items"][slotId] = {
				["icon"] = itemIcon,
				["rarity"] = rarity,
				["link"] = itemLink,
				["searchName"] = searchName
			}
			freeSlots = freeSlots - 1
		else
			gBank["tab-"..tabID]["items"][slotId] = nil
		end
		if itemCount > 1 then
			gBank["tab-"..tabID]["counts"][slotId] = itemCount
		end
	end
	gBank["tab-"..tabID]["freeSlots"] = freeSlots
end

function getGuildInfo(charIndex)
	if AltVaultDB == nil then return end
	local bkgR, bkgG, bkgB, borderR, borderG, borderB, emblemR, emblemG, emblemB, emblemFilename = GetGuildLogoInfo();
	local guildInfo = {}
	local tabardInfo = nil
	if bkgR ~= nil then
		tabardInfo = {
			["bgR"] = bkgR / 255,
			["bgG"] = bkgG / 255,
			["bgB"] = bkgB / 255,
			["borderR"] = borderR / 255,
			["borderG"] = borderG / 255,
			["borderB"] = borderB / 255,
			["emblemR"] = emblemR / 255,
			["emblemG"] = emblemG / 255,
			["emblemB"] = emblemB / 255,
			["emblemTexture"] = emblemFilename
		}
	end
	if IsInGuild() then
		local guildName, guildRank = GetGuildInfo("player")
		local numGuildies = GetNumGuildMembers()
		-- while numGuildies == 0 do numGuildies = GetNumGuildMembers() end
		local _, _, standingID = GetGuildFactionInfo();
		guildInfo = {
			["name"] = guildName,
			["rank"] = guildRank,
			["members"] = numGuildies,
			["tabard"] = tabardInfo,
			["standingID"] = standingID
		}
	else
		guildInfo = {}
	end
	updatePlayerData(charIndex, 'guild', nil, guildInfo)
end

--------------------------------------
-- Get MPlus Data
--------------------------------------
function GetMPlusData(charID)
	local currentSeason = C_MythicPlus.GetCurrentSeason()
	local currentKeystoneLevel = C_MythicPlus.GetOwnedKeystoneLevel()
	local currentKeystoneMap = C_MythicPlus.GetOwnedKeystoneChallengeMapID()
	local dungeonScore = C_ChallengeMode.GetOverallDungeonScore()
	
	local mplusActivities = C_WeeklyRewards.GetActivities(Enum.WeeklyRewardChestThresholdType.MythicPlus)
	local finalActivity = mplusActivities[1]

	local untilReset = C_DateAndTime.GetSecondsUntilWeeklyReset()
	local now = C_DateAndTime.GetCurrentCalendarTime()
	local dataExpires = C_DateAndTime.AdjustTimeByMinutes(now, untilReset / 60)

	local mapScores = {}

	for index, mapID in pairs(C_ChallengeMode.GetMapTable()) do
		local mapName = C_ChallengeMode.GetMapUIInfo(mapID)
		local inTimeInfo, overtimeInfo = C_MythicPlus.GetSeasonBestForMap(mapID)
		local _, overAllScore = C_MythicPlus.GetSeasonBestAffixScoreInfoForMap(mapID);
		local level = 0;
		local dungeonScore = 0;
		if(inTimeInfo and overtimeInfo) then
			local inTimeScoreIsBetter = inTimeInfo.dungeonScore > overtimeInfo.dungeonScore;
			level = inTimeScoreIsBetter and inTimeInfo.level or overtimeInfo.level;
			dungeonScore = inTimeScoreIsBetter and inTimeInfo.dungeonScore or overtimeInfo.dungeonScore;
        elseif(inTimeInfo or overtimeInfo) then
			level = inTimeInfo and inTimeInfo.level or overtimeInfo.level;
			dungeonScore = inTimeInfo and inTimeInfo.dungeonScore or overtimeInfo.dungeonScore;
		end
		mapScores[mapID] = {
			['name'] = mapName,
			['score'] = dungeonScore,
			['overAllScore'] = overAllScore,
			['level'] = level
		}
	end

	local progress = 0
	local threshold = 8

	if finalActivity then
		progress = finalActivity.progress
		threshold = finalActivity.threshold
	end
	
	AltVaultDB.characters[charID]["mplus"] = {
		["currentSeason"] = currentSeason,
		["currentKeystoneLevel"] = currentKeystoneLevel,
		["currentKeystoneMap"] = currentKeystoneMap,
		["dungeonScore"] = dungeonScore,
		["bestScores"] = mapScores,
		["vaultProgress"] = progress,
		["vaultThreshold"] = threshold,
		["dataExpires"] = dataExpires
	}

end

function updatePlayerData(player, category, key, value)
	if not AltVaultDB or not player then return end
	if key ~= nil then
		AltVaultDB.characters[player][category][key] = value
	else
		AltVaultDB.characters[player][category] = value
	end
end

function updateData(key, value)
	AltVaultDB.data[key] = value
end

function core:init(event, ...)
	local charID
	if event == 'ADDON_LOADED' and ... == "AltVault" then
		----------------------------------
		-- Register Slash Commands!
		----------------------------------
		SLASH_RELOADUI1 = "/rl"; -- new slash command for reloading UI
		SlashCmdList.RELOADUI = ReloadUI;

		SLASH_AltVault1 = "/av";
		SlashCmdList.AltVault = OpenAltVault;
	elseif event == "PLAYER_LOGIN" then
		if not AltVaultDB then
			AltVaultDB = {}
			AltVaultDB["data"] = {
				["tokenPrice"] = nil,
				["tokenPriceUpdated"] = nil,
				["shownIntro"] = false,
				["version"] = CURRENT_VERSION
			}
			AltVaultDB["characters"] = {}
		end

		core.initDB()
		core:Print("Welcome to AltVault!")
		core:Print("view your alts with /av or by clicking the minimap icon")
	elseif event == "PLAYER_EQUIPMENT_CHANGED" then
		if AltVaultDB == nil then return end
		charID = charInDB(UnitGUID("player"));
		updatePlayerData(charID, 'gear', nil, GetGearInfo())
		setItemLevelInfo(charID)
	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		charID = charInDB(UnitGUID("player"));
		if charID == false then return end
		local specID, specName, _, specBackground, role = GetSpecializationInfo(GetSpecialization(), false, false, false);
		updatePlayerData(charID, 'character', 'specID', specID)
		updatePlayerData(charID, 'character', 'specName', specName)
		updatePlayerData(charID, 'character', 'specBackground', specBackground)
		updatePlayerData(charID, 'character', 'role', role)
	elseif event == 'BANKFRAME_OPENED' or event == 'BANKFRAME_CLOSED' then
		charID = charInDB(UnitGUID("player"));
		for bankSlotID = NUM_TOTAL_EQUIPPED_BAG_SLOTS + 1, NUM_TOTAL_EQUIPPED_BAG_SLOTS + NUM_BANKBAGSLOTS do
			if AltVaultDB.characters[charID]["bank"]['bag-'..bankSlotID+1] == nil then
				AltVaultDB.characters[charID]["bank"]['bag-'..bankSlotID+1] = {}
			end
			GetBagInfo(bankSlotID, charID)
		end
		GetBagInfo(BANK_CONTAINER, charID)
		if IsReagentBankUnlocked() then
			GetBagInfo(REAGENTBANK_CONTAINER, charID)
		end
		AltVaultDB.characters[charID]["character"]["bankWarned"] = true
	elseif event == "GUILDBANKBAGSLOTS_CHANGED" then
		charID = charInDB(UnitGUID("player"));
		if AltVaultDB.characters[charID]["gbank"] == nil then
			AltVaultDB.characters[charID]["gbank"] = {}
		end
		getGuildBankInfo(charID)
	elseif event == 'BAG_UPDATE' then
		if AltVaultDB == nil then return end
		charID = charInDB(UnitGUID("player"));
		if AltVaultDB.characters[charID] == nil then return end
		for bagID = 0, NUM_TOTAL_EQUIPPED_BAG_SLOTS do
			if AltVaultDB.characters[charID].bags['bag-'..bagID+1] == nil then
				AltVaultDB.characters[charID].bags['bag-'..bagID+1] = {}
			end
			GetBagInfo(bagID, charID)
		end
		setItemLevelInfo(charID)
	elseif event == 'UPDATE_PENDING_MAIL' then
		local hasMail = HasNewMail()
		charID = charInDB(UnitGUID("player"));
		updatePlayerData(charID, 'character', 'hasMail', hasMail)
	elseif event == "PLAYER_MONEY" then
		charID = charInDB(UnitGUID("player"));
		updatePlayerData(charID, 'character', 'money', GetMoney())
	elseif event == "AUCTION_HOUSE_SHOW" or event == "AUCTION_HOUSE_CLOSED" then
		local marketPrice = C_WowTokenPublic.GetCurrentMarketPrice()
		local currentTime = C_DateAndTime.GetCurrentCalendarTime()
		updateData('tokenPrice', marketPrice)
		updateData('tokenPriceUpdated', currentTime)
	elseif event == "GUILD_ROSTER_UPDATE" or event == "PLAYER_GUILD_UPDATE" then
		charID = charInDB(UnitGUID("player"));
		getGuildInfo(charID)
	elseif event == "TIME_PLAYED_MSG" then
		local total, level = ...
		charID = charInDB(UnitGUID("player"));
		updatePlayerData(charID, 'character', 'played', total)
		updatePlayerData(charID, 'character', 'playedAtLevel', level)
	elseif event == "PLAYER_LEAVING_WORLD" then
		charID = charInDB(UnitGUID("player"));
		local currentTime = C_DateAndTime.GetCurrentCalendarTime()
		updatePlayerData(charID, 'character', 'lastPlayed', currentTime)
	elseif event == 'CHALLENGE_MODE_LEADERS_UPDATE' then
		charID = charInDB(UnitGUID("player"))
		GetMPlusData(charID)
	end
end

local AltVaultEvents = CreateFrame("Frame");

AltVaultEvents:RegisterEvent("ADDON_LOADED");
AltVaultEvents:RegisterEvent("PLAYER_LOGIN");
AltVaultEvents:RegisterEvent("PLAYER_LEAVING_WORLD");
AltVaultEvents:RegisterEvent("UPDATE_PENDING_MAIL");
AltVaultEvents:RegisterEvent("PLAYER_MONEY");
AltVaultEvents:RegisterEvent("AUCTION_HOUSE_SHOW");
AltVaultEvents:RegisterEvent("AUCTION_HOUSE_CLOSED");
AltVaultEvents:RegisterEvent("PLAYER_GUILD_UPDATE");
AltVaultEvents:RegisterEvent("PLAYER_ALIVE");
AltVaultEvents:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
AltVaultEvents:RegisterEvent("HONOR_XP_UPDATE");
AltVaultEvents:RegisterEvent("TIME_PLAYED_MSG");
AltVaultEvents:RegisterEvent("BANKFRAME_OPENED");
AltVaultEvents:RegisterEvent("BANKFRAME_CLOSED");
AltVaultEvents:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED");
AltVaultEvents:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
AltVaultEvents:RegisterEvent("BAG_UPDATE");
AltVaultEvents:RegisterEvent("GUILD_ROSTER_UPDATE");
AltVaultEvents:RegisterEvent("CHALLENGE_MODE_LEADERS_UPDATE");



AltVaultEvents:SetScript("OnEvent", core.init);